import QtQuick 2.4

Item {
    id: root

    width: childrenRect.width
    height: childrenRect.height

    property string label: ""
    property string upimage: ""
    property string downimage: ""
    property string disabledimage: ""
    property bool toggle: false
    property bool repeat: false
    property int repeatInterval: 100
    property alias fontsize: label.font.pixelSize
    property alias font: label.font
    property bool label_under
    property bool label_visible: true
    property alias label_color: label.color
    property bool isDown: false
    property string icon
    property int icon_x_offset: 0
    property int icon_y_offset: 0
    property int label_x_offset: 0
    property int label_y_offset: 0
    property int _repeatDelay: 0
    property bool useParentContainerSize: false
    property real mouseAreaScale: 1.0
    property real labelMouseAreaScale: 1.0
    property bool preventStealing: true
    property bool labelClickable: false
    property list<Item> radioGroup
    property bool beep: true
    property int wrapWidth: 0
    property int reenableDelay: 0
    property bool _lastPressLegit: true

    signal clicked()
    signal pressAndHold()

    onClicked: {
        if (!repeat && reenableDelay > 0) {
            disabledTimer.start()
        }
    }

    // call to set the state down (true) or up for a toggle button
    function setDown(is_down) {
        isDown = is_down
    }

    onIsDownChanged: {
        if(!isDown)
            return
        if(radioGroup) {
            for (var i=0 ; i < radioGroup.length ; i++ ) {
                var item = radioGroup[i]
                if(item != root)
                    item.setDown(false)
            }
        }
    }

    Timer {
        id: repeat_timer

        onRunningChanged: _repeatDelay = 0

        interval: repeatInterval
        repeat: true

        onTriggered: {
            // delay the onset of repeat
            if (_repeatDelay < 6)
            {
                _repeatDelay++
                return
            }
            if (root.isDown && !root.toggle)
            {
                clicked()
            }
        }
    }

    Timer {
        id: disabledTimer

        interval: reenableDelay
        repeat: false
    }

    Image {
        id: button_image

        source: root.enabled ? (isDown ? downimage: upimage) : (root.disabledimage ? root.disabledimage : upimage)
        width: if(useParentContainerSize) parent.width
        height: if(useParentContainerSize) parent.height

        fillMode: "PreserveAspectFit"
        smooth: true

        // make the button a bit transparent when disabled
        opacity: root.disabledimage ? 1.0 : (enabled ? 1 : 0.3)

        Rectangle{
            id: stimArea

            anchors.fill: parent
            color: "transparent"
            opacity:0.3
            scale: mouseAreaScale

            MouseArea{
                preventStealing: root.preventStealing
                anchors.fill: parent
                onPressed: {
                    if (disabledTimer.running) {
                        root._lastPressLegit = false
                        return
                    }

                    if( root.toggle ) {
                        // if the button is in a radio group, only toggle down on click
                        if((radioGroup.length == 0) || !isDown)
                            root.isDown = !root.isDown
                    }
                    else if (root.repeat) {
                        repeat_timer.running = true
                        root.isDown = true;
                    }
                    else
                        root.isDown = !root.isDown

                    root._lastPressLegit = true
                }

                onReleased: {
                    if (disabledTimer.running || !root._lastPressLegit)
                        return

                    if (!root.toggle)
                        root.isDown = false;

                    if( containsMouse && _repeatDelay <= 5 && !longHoldTimer.triggering)
                    {
                        root.clicked()
                    }
                    longHoldTimer.triggering = false
                    repeat_timer.running = false
                }

                onPressAndHold: {
                    if (disabledTimer.running)
                        return

                    // Mouse area press and hold time is 800ms.
                    // Secondary timer allows longer hold time for event
                    longHoldTimer.start()
                }

                Timer {
                    id: longHoldTimer

                    property bool triggering: false
                    running: false
                    repeat: false
                    interval: 2200

                    // trigger a 'pressAndHold' event if button is still down after 3 seconds
                    onTriggered: {
                        if( !root.isDown )
                            return
                        triggering = true
                        root.pressAndHold()
                    }
                }
            } // end MouseArea
        } // end stimArea

        // an icon image may go on top of the button image
        Image {
            id: icon_image

            source: icon ? icon : ""
            x: icon_x_offset
            y: icon_y_offset
            smooth: true
            anchors {
                horizontalCenter: parent.horizontalCenter
                horizontalCenterOffset: icon_x_offset
                verticalCenter: parent.verticalCenter
                verticalCenterOffset: icon_y_offset
            }
            fillMode: "PreserveAspectFit"
        }
    }

    Text {
        id: label

        visible: label_visible
        width: root.wrapWidth > 0 ? root.wrapWidth : undefined
        opacity: enabled ? 1 : 0.3

        anchors {
            horizontalCenter:  button_image.horizontalCenter
            verticalCenter: button_image.verticalCenter
            horizontalCenterOffset: root.label_x_offset
            // if label_under is set, the reference point is the bottom of the button image
            verticalCenterOffset: label_under ? button_image.height / 2 + label_y_offset : label_y_offset
        }
        font {
            bold: true
            pixelSize: 21
        }

        color: "white"
        text: parent.label

        verticalAlignment: Text.AlignVCenter
        horizontalAlignment: Text.AlignHCenter

        wrapMode: root.wrapWidth > 0 ? Text.WordWrap : Text.NoWrap
        MouseArea {
            enabled: root.labelClickable
            anchors.fill: parent
            onClicked: if (!disabledTimer.running) root.clicked()
            scale: labelMouseAreaScale
        }
    }
}
